"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.models.components import domainschema as components_domainschema
from dub.types import BaseModel
from dub.utils import FieldMetadata, QueryParamMetadata
import pydantic
from typing import Callable, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListDomainsRequestTypedDict(TypedDict):
    archived: NotRequired[bool]
    r"""Whether to include archived domains in the response. Defaults to `false` if not provided."""
    search: NotRequired[str]
    r"""The search term to filter the domains by."""
    page: NotRequired[float]
    r"""The page number for pagination."""
    page_size: NotRequired[float]
    r"""The number of items per page."""


class ListDomainsRequest(BaseModel):
    archived: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = True
    r"""Whether to include archived domains in the response. Defaults to `false` if not provided."""

    search: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The search term to filter the domains by."""

    page: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""The page number for pagination."""

    page_size: Annotated[
        Optional[float],
        pydantic.Field(alias="pageSize"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 50
    r"""The number of items per page."""


class ListDomainsResponseTypedDict(TypedDict):
    result: List[components_domainschema.DomainSchemaTypedDict]


class ListDomainsResponse(BaseModel):
    next: Callable[[], Optional[ListDomainsResponse]]

    result: List[components_domainschema.DomainSchema]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
from enum import Enum
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Trigger(str, Enum):
    r"""The type of trigger method: link click or QR scan"""

    QR = "qr"
    LINK = "link"


class AnalyticsTriggersTypedDict(TypedDict):
    trigger: Trigger
    r"""The type of trigger method: link click or QR scan"""
    clicks: NotRequired[float]
    r"""The number of clicks from this trigger method"""
    leads: NotRequired[float]
    r"""The number of leads from this trigger method"""
    sales: NotRequired[float]
    r"""The number of sales from this trigger method"""
    sale_amount: NotRequired[float]
    r"""The total amount of sales from this trigger method, in cents"""


class AnalyticsTriggers(BaseModel):
    trigger: Trigger
    r"""The type of trigger method: link click or QR scan"""

    clicks: Optional[float] = 0
    r"""The number of clicks from this trigger method"""

    leads: Optional[float] = 0
    r"""The number of leads from this trigger method"""

    sales: Optional[float] = 0
    r"""The number of sales from this trigger method"""

    sale_amount: Annotated[Optional[float], pydantic.Field(alias="saleAmount")] = 0
    r"""The total amount of sales from this trigger method, in cents"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LinkGeoTargetingTypedDict(TypedDict):
    r"""Geo targeting information for the short link in JSON format `{[COUNTRY]: https://example.com }`."""

    af: NotRequired[str]
    al: NotRequired[str]
    dz: NotRequired[str]
    as_: NotRequired[str]
    ad: NotRequired[str]
    ao: NotRequired[str]
    ai: NotRequired[str]
    aq: NotRequired[str]
    ag: NotRequired[str]
    ar: NotRequired[str]
    am: NotRequired[str]
    aw: NotRequired[str]
    au: NotRequired[str]
    at: NotRequired[str]
    az: NotRequired[str]
    bs: NotRequired[str]
    bh: NotRequired[str]
    bd: NotRequired[str]
    bb: NotRequired[str]
    by: NotRequired[str]
    be: NotRequired[str]
    bz: NotRequired[str]
    bj: NotRequired[str]
    bm: NotRequired[str]
    bt: NotRequired[str]
    bo: NotRequired[str]
    ba: NotRequired[str]
    bw: NotRequired[str]
    bv: NotRequired[str]
    br: NotRequired[str]
    io: NotRequired[str]
    bn: NotRequired[str]
    bg: NotRequired[str]
    bf: NotRequired[str]
    bi: NotRequired[str]
    kh: NotRequired[str]
    cm: NotRequired[str]
    ca: NotRequired[str]
    cv: NotRequired[str]
    ky: NotRequired[str]
    cf: NotRequired[str]
    td: NotRequired[str]
    cl: NotRequired[str]
    cn: NotRequired[str]
    cx: NotRequired[str]
    cc: NotRequired[str]
    co: NotRequired[str]
    km: NotRequired[str]
    cg: NotRequired[str]
    cd: NotRequired[str]
    ck: NotRequired[str]
    cr: NotRequired[str]
    ci: NotRequired[str]
    hr: NotRequired[str]
    cu: NotRequired[str]
    cy: NotRequired[str]
    cz: NotRequired[str]
    dk: NotRequired[str]
    dj: NotRequired[str]
    dm: NotRequired[str]
    do: NotRequired[str]
    ec: NotRequired[str]
    eg: NotRequired[str]
    sv: NotRequired[str]
    gq: NotRequired[str]
    er: NotRequired[str]
    ee: NotRequired[str]
    et: NotRequired[str]
    fk: NotRequired[str]
    fo: NotRequired[str]
    fj: NotRequired[str]
    fi: NotRequired[str]
    fr: NotRequired[str]
    gf: NotRequired[str]
    pf: NotRequired[str]
    tf: NotRequired[str]
    ga: NotRequired[str]
    gm: NotRequired[str]
    ge: NotRequired[str]
    de: NotRequired[str]
    gh: NotRequired[str]
    gi: NotRequired[str]
    gr: NotRequired[str]
    gl: NotRequired[str]
    gd: NotRequired[str]
    gp: NotRequired[str]
    gu: NotRequired[str]
    gt: NotRequired[str]
    gn: NotRequired[str]
    gw: NotRequired[str]
    gy: NotRequired[str]
    ht: NotRequired[str]
    hm: NotRequired[str]
    va: NotRequired[str]
    hn: NotRequired[str]
    hk: NotRequired[str]
    hu: NotRequired[str]
    is_: NotRequired[str]
    in_: NotRequired[str]
    id: NotRequired[str]
    ir: NotRequired[str]
    iq: NotRequired[str]
    ie: NotRequired[str]
    il: NotRequired[str]
    it: NotRequired[str]
    jm: NotRequired[str]
    jp: NotRequired[str]
    jo: NotRequired[str]
    kz: NotRequired[str]
    ke: NotRequired[str]
    ki: NotRequired[str]
    kp: NotRequired[str]
    kr: NotRequired[str]
    kw: NotRequired[str]
    kg: NotRequired[str]
    la: NotRequired[str]
    lv: NotRequired[str]
    lb: NotRequired[str]
    ls: NotRequired[str]
    lr: NotRequired[str]
    ly: NotRequired[str]
    li: NotRequired[str]
    lt: NotRequired[str]
    lu: NotRequired[str]
    mo: NotRequired[str]
    mg: NotRequired[str]
    mw: NotRequired[str]
    my: NotRequired[str]
    mv: NotRequired[str]
    ml: NotRequired[str]
    mt: NotRequired[str]
    mh: NotRequired[str]
    mq: NotRequired[str]
    mr: NotRequired[str]
    mu: NotRequired[str]
    yt: NotRequired[str]
    mx: NotRequired[str]
    fm: NotRequired[str]
    md: NotRequired[str]
    mc: NotRequired[str]
    mn: NotRequired[str]
    ms: NotRequired[str]
    ma: NotRequired[str]
    mz: NotRequired[str]
    mm: NotRequired[str]
    na: NotRequired[str]
    nr: NotRequired[str]
    np: NotRequired[str]
    nl: NotRequired[str]
    nc: NotRequired[str]
    nz: NotRequired[str]
    ni: NotRequired[str]
    ne: NotRequired[str]
    ng: NotRequired[str]
    nu: NotRequired[str]
    nf: NotRequired[str]
    mk: NotRequired[str]
    mp: NotRequired[str]
    no: NotRequired[str]
    om: NotRequired[str]
    pk: NotRequired[str]
    pw: NotRequired[str]
    ps: NotRequired[str]
    pa: NotRequired[str]
    pg: NotRequired[str]
    py: NotRequired[str]
    pe: NotRequired[str]
    ph: NotRequired[str]
    pn: NotRequired[str]
    pl: NotRequired[str]
    pt: NotRequired[str]
    pr: NotRequired[str]
    qa: NotRequired[str]
    re: NotRequired[str]
    ro: NotRequired[str]
    ru: NotRequired[str]
    rw: NotRequired[str]
    sh: NotRequired[str]
    kn: NotRequired[str]
    lc: NotRequired[str]
    pm: NotRequired[str]
    vc: NotRequired[str]
    ws: NotRequired[str]
    sm: NotRequired[str]
    st: NotRequired[str]
    sa: NotRequired[str]
    sn: NotRequired[str]
    sc: NotRequired[str]
    sl: NotRequired[str]
    sg: NotRequired[str]
    sk: NotRequired[str]
    si: NotRequired[str]
    sb: NotRequired[str]
    so: NotRequired[str]
    za: NotRequired[str]
    gs: NotRequired[str]
    es: NotRequired[str]
    lk: NotRequired[str]
    sd: NotRequired[str]
    sr: NotRequired[str]
    sj: NotRequired[str]
    sz: NotRequired[str]
    se: NotRequired[str]
    ch: NotRequired[str]
    sy: NotRequired[str]
    tw: NotRequired[str]
    tj: NotRequired[str]
    tz: NotRequired[str]
    th: NotRequired[str]
    tl: NotRequired[str]
    tg: NotRequired[str]
    tk: NotRequired[str]
    to: NotRequired[str]
    tt: NotRequired[str]
    tn: NotRequired[str]
    tr: NotRequired[str]
    tm: NotRequired[str]
    tc: NotRequired[str]
    tv: NotRequired[str]
    ug: NotRequired[str]
    ua: NotRequired[str]
    ae: NotRequired[str]
    gb: NotRequired[str]
    us: NotRequired[str]
    um: NotRequired[str]
    uy: NotRequired[str]
    uz: NotRequired[str]
    vu: NotRequired[str]
    ve: NotRequired[str]
    vn: NotRequired[str]
    vg: NotRequired[str]
    vi: NotRequired[str]
    wf: NotRequired[str]
    eh: NotRequired[str]
    ye: NotRequired[str]
    zm: NotRequired[str]
    zw: NotRequired[str]
    ax: NotRequired[str]
    bq: NotRequired[str]
    cw: NotRequired[str]
    gg: NotRequired[str]
    im: NotRequired[str]
    je: NotRequired[str]
    me: NotRequired[str]
    bl: NotRequired[str]
    mf: NotRequired[str]
    rs: NotRequired[str]
    sx: NotRequired[str]
    ss: NotRequired[str]
    xk: NotRequired[str]


class LinkGeoTargeting(BaseModel):
    r"""Geo targeting information for the short link in JSON format `{[COUNTRY]: https://example.com }`."""

    af: Annotated[Optional[str], pydantic.Field(alias="AF")] = None

    al: Annotated[Optional[str], pydantic.Field(alias="AL")] = None

    dz: Annotated[Optional[str], pydantic.Field(alias="DZ")] = None

    as_: Annotated[Optional[str], pydantic.Field(alias="AS")] = None

    ad: Annotated[Optional[str], pydantic.Field(alias="AD")] = None

    ao: Annotated[Optional[str], pydantic.Field(alias="AO")] = None

    ai: Annotated[Optional[str], pydantic.Field(alias="AI")] = None

    aq: Annotated[Optional[str], pydantic.Field(alias="AQ")] = None

    ag: Annotated[Optional[str], pydantic.Field(alias="AG")] = None

    ar: Annotated[Optional[str], pydantic.Field(alias="AR")] = None

    am: Annotated[Optional[str], pydantic.Field(alias="AM")] = None

    aw: Annotated[Optional[str], pydantic.Field(alias="AW")] = None

    au: Annotated[Optional[str], pydantic.Field(alias="AU")] = None

    at: Annotated[Optional[str], pydantic.Field(alias="AT")] = None

    az: Annotated[Optional[str], pydantic.Field(alias="AZ")] = None

    bs: Annotated[Optional[str], pydantic.Field(alias="BS")] = None

    bh: Annotated[Optional[str], pydantic.Field(alias="BH")] = None

    bd: Annotated[Optional[str], pydantic.Field(alias="BD")] = None

    bb: Annotated[Optional[str], pydantic.Field(alias="BB")] = None

    by: Annotated[Optional[str], pydantic.Field(alias="BY")] = None

    be: Annotated[Optional[str], pydantic.Field(alias="BE")] = None

    bz: Annotated[Optional[str], pydantic.Field(alias="BZ")] = None

    bj: Annotated[Optional[str], pydantic.Field(alias="BJ")] = None

    bm: Annotated[Optional[str], pydantic.Field(alias="BM")] = None

    bt: Annotated[Optional[str], pydantic.Field(alias="BT")] = None

    bo: Annotated[Optional[str], pydantic.Field(alias="BO")] = None

    ba: Annotated[Optional[str], pydantic.Field(alias="BA")] = None

    bw: Annotated[Optional[str], pydantic.Field(alias="BW")] = None

    bv: Annotated[Optional[str], pydantic.Field(alias="BV")] = None

    br: Annotated[Optional[str], pydantic.Field(alias="BR")] = None

    io: Annotated[Optional[str], pydantic.Field(alias="IO")] = None

    bn: Annotated[Optional[str], pydantic.Field(alias="BN")] = None

    bg: Annotated[Optional[str], pydantic.Field(alias="BG")] = None

    bf: Annotated[Optional[str], pydantic.Field(alias="BF")] = None

    bi: Annotated[Optional[str], pydantic.Field(alias="BI")] = None

    kh: Annotated[Optional[str], pydantic.Field(alias="KH")] = None

    cm: Annotated[Optional[str], pydantic.Field(alias="CM")] = None

    ca: Annotated[Optional[str], pydantic.Field(alias="CA")] = None

    cv: Annotated[Optional[str], pydantic.Field(alias="CV")] = None

    ky: Annotated[Optional[str], pydantic.Field(alias="KY")] = None

    cf: Annotated[Optional[str], pydantic.Field(alias="CF")] = None

    td: Annotated[Optional[str], pydantic.Field(alias="TD")] = None

    cl: Annotated[Optional[str], pydantic.Field(alias="CL")] = None

    cn: Annotated[Optional[str], pydantic.Field(alias="CN")] = None

    cx: Annotated[Optional[str], pydantic.Field(alias="CX")] = None

    cc: Annotated[Optional[str], pydantic.Field(alias="CC")] = None

    co: Annotated[Optional[str], pydantic.Field(alias="CO")] = None

    km: Annotated[Optional[str], pydantic.Field(alias="KM")] = None

    cg: Annotated[Optional[str], pydantic.Field(alias="CG")] = None

    cd: Annotated[Optional[str], pydantic.Field(alias="CD")] = None

    ck: Annotated[Optional[str], pydantic.Field(alias="CK")] = None

    cr: Annotated[Optional[str], pydantic.Field(alias="CR")] = None

    ci: Annotated[Optional[str], pydantic.Field(alias="CI")] = None

    hr: Annotated[Optional[str], pydantic.Field(alias="HR")] = None

    cu: Annotated[Optional[str], pydantic.Field(alias="CU")] = None

    cy: Annotated[Optional[str], pydantic.Field(alias="CY")] = None

    cz: Annotated[Optional[str], pydantic.Field(alias="CZ")] = None

    dk: Annotated[Optional[str], pydantic.Field(alias="DK")] = None

    dj: Annotated[Optional[str], pydantic.Field(alias="DJ")] = None

    dm: Annotated[Optional[str], pydantic.Field(alias="DM")] = None

    do: Annotated[Optional[str], pydantic.Field(alias="DO")] = None

    ec: Annotated[Optional[str], pydantic.Field(alias="EC")] = None

    eg: Annotated[Optional[str], pydantic.Field(alias="EG")] = None

    sv: Annotated[Optional[str], pydantic.Field(alias="SV")] = None

    gq: Annotated[Optional[str], pydantic.Field(alias="GQ")] = None

    er: Annotated[Optional[str], pydantic.Field(alias="ER")] = None

    ee: Annotated[Optional[str], pydantic.Field(alias="EE")] = None

    et: Annotated[Optional[str], pydantic.Field(alias="ET")] = None

    fk: Annotated[Optional[str], pydantic.Field(alias="FK")] = None

    fo: Annotated[Optional[str], pydantic.Field(alias="FO")] = None

    fj: Annotated[Optional[str], pydantic.Field(alias="FJ")] = None

    fi: Annotated[Optional[str], pydantic.Field(alias="FI")] = None

    fr: Annotated[Optional[str], pydantic.Field(alias="FR")] = None

    gf: Annotated[Optional[str], pydantic.Field(alias="GF")] = None

    pf: Annotated[Optional[str], pydantic.Field(alias="PF")] = None

    tf: Annotated[Optional[str], pydantic.Field(alias="TF")] = None

    ga: Annotated[Optional[str], pydantic.Field(alias="GA")] = None

    gm: Annotated[Optional[str], pydantic.Field(alias="GM")] = None

    ge: Annotated[Optional[str], pydantic.Field(alias="GE")] = None

    de: Annotated[Optional[str], pydantic.Field(alias="DE")] = None

    gh: Annotated[Optional[str], pydantic.Field(alias="GH")] = None

    gi: Annotated[Optional[str], pydantic.Field(alias="GI")] = None

    gr: Annotated[Optional[str], pydantic.Field(alias="GR")] = None

    gl: Annotated[Optional[str], pydantic.Field(alias="GL")] = None

    gd: Annotated[Optional[str], pydantic.Field(alias="GD")] = None

    gp: Annotated[Optional[str], pydantic.Field(alias="GP")] = None

    gu: Annotated[Optional[str], pydantic.Field(alias="GU")] = None

    gt: Annotated[Optional[str], pydantic.Field(alias="GT")] = None

    gn: Annotated[Optional[str], pydantic.Field(alias="GN")] = None

    gw: Annotated[Optional[str], pydantic.Field(alias="GW")] = None

    gy: Annotated[Optional[str], pydantic.Field(alias="GY")] = None

    ht: Annotated[Optional[str], pydantic.Field(alias="HT")] = None

    hm: Annotated[Optional[str], pydantic.Field(alias="HM")] = None

    va: Annotated[Optional[str], pydantic.Field(alias="VA")] = None

    hn: Annotated[Optional[str], pydantic.Field(alias="HN")] = None

    hk: Annotated[Optional[str], pydantic.Field(alias="HK")] = None

    hu: Annotated[Optional[str], pydantic.Field(alias="HU")] = None

    is_: Annotated[Optional[str], pydantic.Field(alias="IS")] = None

    in_: Annotated[Optional[str], pydantic.Field(alias="IN")] = None

    id: Annotated[Optional[str], pydantic.Field(alias="ID")] = None

    ir: Annotated[Optional[str], pydantic.Field(alias="IR")] = None

    iq: Annotated[Optional[str], pydantic.Field(alias="IQ")] = None

    ie: Annotated[Optional[str], pydantic.Field(alias="IE")] = None

    il: Annotated[Optional[str], pydantic.Field(alias="IL")] = None

    it: Annotated[Optional[str], pydantic.Field(alias="IT")] = None

    jm: Annotated[Optional[str], pydantic.Field(alias="JM")] = None

    jp: Annotated[Optional[str], pydantic.Field(alias="JP")] = None

    jo: Annotated[Optional[str], pydantic.Field(alias="JO")] = None

    kz: Annotated[Optional[str], pydantic.Field(alias="KZ")] = None

    ke: Annotated[Optional[str], pydantic.Field(alias="KE")] = None

    ki: Annotated[Optional[str], pydantic.Field(alias="KI")] = None

    kp: Annotated[Optional[str], pydantic.Field(alias="KP")] = None

    kr: Annotated[Optional[str], pydantic.Field(alias="KR")] = None

    kw: Annotated[Optional[str], pydantic.Field(alias="KW")] = None

    kg: Annotated[Optional[str], pydantic.Field(alias="KG")] = None

    la: Annotated[Optional[str], pydantic.Field(alias="LA")] = None

    lv: Annotated[Optional[str], pydantic.Field(alias="LV")] = None

    lb: Annotated[Optional[str], pydantic.Field(alias="LB")] = None

    ls: Annotated[Optional[str], pydantic.Field(alias="LS")] = None

    lr: Annotated[Optional[str], pydantic.Field(alias="LR")] = None

    ly: Annotated[Optional[str], pydantic.Field(alias="LY")] = None

    li: Annotated[Optional[str], pydantic.Field(alias="LI")] = None

    lt: Annotated[Optional[str], pydantic.Field(alias="LT")] = None

    lu: Annotated[Optional[str], pydantic.Field(alias="LU")] = None

    mo: Annotated[Optional[str], pydantic.Field(alias="MO")] = None

    mg: Annotated[Optional[str], pydantic.Field(alias="MG")] = None

    mw: Annotated[Optional[str], pydantic.Field(alias="MW")] = None

    my: Annotated[Optional[str], pydantic.Field(alias="MY")] = None

    mv: Annotated[Optional[str], pydantic.Field(alias="MV")] = None

    ml: Annotated[Optional[str], pydantic.Field(alias="ML")] = None

    mt: Annotated[Optional[str], pydantic.Field(alias="MT")] = None

    mh: Annotated[Optional[str], pydantic.Field(alias="MH")] = None

    mq: Annotated[Optional[str], pydantic.Field(alias="MQ")] = None

    mr: Annotated[Optional[str], pydantic.Field(alias="MR")] = None

    mu: Annotated[Optional[str], pydantic.Field(alias="MU")] = None

    yt: Annotated[Optional[str], pydantic.Field(alias="YT")] = None

    mx: Annotated[Optional[str], pydantic.Field(alias="MX")] = None

    fm: Annotated[Optional[str], pydantic.Field(alias="FM")] = None

    md: Annotated[Optional[str], pydantic.Field(alias="MD")] = None

    mc: Annotated[Optional[str], pydantic.Field(alias="MC")] = None

    mn: Annotated[Optional[str], pydantic.Field(alias="MN")] = None

    ms: Annotated[Optional[str], pydantic.Field(alias="MS")] = None

    ma: Annotated[Optional[str], pydantic.Field(alias="MA")] = None

    mz: Annotated[Optional[str], pydantic.Field(alias="MZ")] = None

    mm: Annotated[Optional[str], pydantic.Field(alias="MM")] = None

    na: Annotated[Optional[str], pydantic.Field(alias="NA")] = None

    nr: Annotated[Optional[str], pydantic.Field(alias="NR")] = None

    np: Annotated[Optional[str], pydantic.Field(alias="NP")] = None

    nl: Annotated[Optional[str], pydantic.Field(alias="NL")] = None

    nc: Annotated[Optional[str], pydantic.Field(alias="NC")] = None

    nz: Annotated[Optional[str], pydantic.Field(alias="NZ")] = None

    ni: Annotated[Optional[str], pydantic.Field(alias="NI")] = None

    ne: Annotated[Optional[str], pydantic.Field(alias="NE")] = None

    ng: Annotated[Optional[str], pydantic.Field(alias="NG")] = None

    nu: Annotated[Optional[str], pydantic.Field(alias="NU")] = None

    nf: Annotated[Optional[str], pydantic.Field(alias="NF")] = None

    mk: Annotated[Optional[str], pydantic.Field(alias="MK")] = None

    mp: Annotated[Optional[str], pydantic.Field(alias="MP")] = None

    no: Annotated[Optional[str], pydantic.Field(alias="NO")] = None

    om: Annotated[Optional[str], pydantic.Field(alias="OM")] = None

    pk: Annotated[Optional[str], pydantic.Field(alias="PK")] = None

    pw: Annotated[Optional[str], pydantic.Field(alias="PW")] = None

    ps: Annotated[Optional[str], pydantic.Field(alias="PS")] = None

    pa: Annotated[Optional[str], pydantic.Field(alias="PA")] = None

    pg: Annotated[Optional[str], pydantic.Field(alias="PG")] = None

    py: Annotated[Optional[str], pydantic.Field(alias="PY")] = None

    pe: Annotated[Optional[str], pydantic.Field(alias="PE")] = None

    ph: Annotated[Optional[str], pydantic.Field(alias="PH")] = None

    pn: Annotated[Optional[str], pydantic.Field(alias="PN")] = None

    pl: Annotated[Optional[str], pydantic.Field(alias="PL")] = None

    pt: Annotated[Optional[str], pydantic.Field(alias="PT")] = None

    pr: Annotated[Optional[str], pydantic.Field(alias="PR")] = None

    qa: Annotated[Optional[str], pydantic.Field(alias="QA")] = None

    re: Annotated[Optional[str], pydantic.Field(alias="RE")] = None

    ro: Annotated[Optional[str], pydantic.Field(alias="RO")] = None

    ru: Annotated[Optional[str], pydantic.Field(alias="RU")] = None

    rw: Annotated[Optional[str], pydantic.Field(alias="RW")] = None

    sh: Annotated[Optional[str], pydantic.Field(alias="SH")] = None

    kn: Annotated[Optional[str], pydantic.Field(alias="KN")] = None

    lc: Annotated[Optional[str], pydantic.Field(alias="LC")] = None

    pm: Annotated[Optional[str], pydantic.Field(alias="PM")] = None

    vc: Annotated[Optional[str], pydantic.Field(alias="VC")] = None

    ws: Annotated[Optional[str], pydantic.Field(alias="WS")] = None

    sm: Annotated[Optional[str], pydantic.Field(alias="SM")] = None

    st: Annotated[Optional[str], pydantic.Field(alias="ST")] = None

    sa: Annotated[Optional[str], pydantic.Field(alias="SA")] = None

    sn: Annotated[Optional[str], pydantic.Field(alias="SN")] = None

    sc: Annotated[Optional[str], pydantic.Field(alias="SC")] = None

    sl: Annotated[Optional[str], pydantic.Field(alias="SL")] = None

    sg: Annotated[Optional[str], pydantic.Field(alias="SG")] = None

    sk: Annotated[Optional[str], pydantic.Field(alias="SK")] = None

    si: Annotated[Optional[str], pydantic.Field(alias="SI")] = None

    sb: Annotated[Optional[str], pydantic.Field(alias="SB")] = None

    so: Annotated[Optional[str], pydantic.Field(alias="SO")] = None

    za: Annotated[Optional[str], pydantic.Field(alias="ZA")] = None

    gs: Annotated[Optional[str], pydantic.Field(alias="GS")] = None

    es: Annotated[Optional[str], pydantic.Field(alias="ES")] = None

    lk: Annotated[Optional[str], pydantic.Field(alias="LK")] = None

    sd: Annotated[Optional[str], pydantic.Field(alias="SD")] = None

    sr: Annotated[Optional[str], pydantic.Field(alias="SR")] = None

    sj: Annotated[Optional[str], pydantic.Field(alias="SJ")] = None

    sz: Annotated[Optional[str], pydantic.Field(alias="SZ")] = None

    se: Annotated[Optional[str], pydantic.Field(alias="SE")] = None

    ch: Annotated[Optional[str], pydantic.Field(alias="CH")] = None

    sy: Annotated[Optional[str], pydantic.Field(alias="SY")] = None

    tw: Annotated[Optional[str], pydantic.Field(alias="TW")] = None

    tj: Annotated[Optional[str], pydantic.Field(alias="TJ")] = None

    tz: Annotated[Optional[str], pydantic.Field(alias="TZ")] = None

    th: Annotated[Optional[str], pydantic.Field(alias="TH")] = None

    tl: Annotated[Optional[str], pydantic.Field(alias="TL")] = None

    tg: Annotated[Optional[str], pydantic.Field(alias="TG")] = None

    tk: Annotated[Optional[str], pydantic.Field(alias="TK")] = None

    to: Annotated[Optional[str], pydantic.Field(alias="TO")] = None

    tt: Annotated[Optional[str], pydantic.Field(alias="TT")] = None

    tn: Annotated[Optional[str], pydantic.Field(alias="TN")] = None

    tr: Annotated[Optional[str], pydantic.Field(alias="TR")] = None

    tm: Annotated[Optional[str], pydantic.Field(alias="TM")] = None

    tc: Annotated[Optional[str], pydantic.Field(alias="TC")] = None

    tv: Annotated[Optional[str], pydantic.Field(alias="TV")] = None

    ug: Annotated[Optional[str], pydantic.Field(alias="UG")] = None

    ua: Annotated[Optional[str], pydantic.Field(alias="UA")] = None

    ae: Annotated[Optional[str], pydantic.Field(alias="AE")] = None

    gb: Annotated[Optional[str], pydantic.Field(alias="GB")] = None

    us: Annotated[Optional[str], pydantic.Field(alias="US")] = None

    um: Annotated[Optional[str], pydantic.Field(alias="UM")] = None

    uy: Annotated[Optional[str], pydantic.Field(alias="UY")] = None

    uz: Annotated[Optional[str], pydantic.Field(alias="UZ")] = None

    vu: Annotated[Optional[str], pydantic.Field(alias="VU")] = None

    ve: Annotated[Optional[str], pydantic.Field(alias="VE")] = None

    vn: Annotated[Optional[str], pydantic.Field(alias="VN")] = None

    vg: Annotated[Optional[str], pydantic.Field(alias="VG")] = None

    vi: Annotated[Optional[str], pydantic.Field(alias="VI")] = None

    wf: Annotated[Optional[str], pydantic.Field(alias="WF")] = None

    eh: Annotated[Optional[str], pydantic.Field(alias="EH")] = None

    ye: Annotated[Optional[str], pydantic.Field(alias="YE")] = None

    zm: Annotated[Optional[str], pydantic.Field(alias="ZM")] = None

    zw: Annotated[Optional[str], pydantic.Field(alias="ZW")] = None

    ax: Annotated[Optional[str], pydantic.Field(alias="AX")] = None

    bq: Annotated[Optional[str], pydantic.Field(alias="BQ")] = None

    cw: Annotated[Optional[str], pydantic.Field(alias="CW")] = None

    gg: Annotated[Optional[str], pydantic.Field(alias="GG")] = None

    im: Annotated[Optional[str], pydantic.Field(alias="IM")] = None

    je: Annotated[Optional[str], pydantic.Field(alias="JE")] = None

    me: Annotated[Optional[str], pydantic.Field(alias="ME")] = None

    bl: Annotated[Optional[str], pydantic.Field(alias="BL")] = None

    mf: Annotated[Optional[str], pydantic.Field(alias="MF")] = None

    rs: Annotated[Optional[str], pydantic.Field(alias="RS")] = None

    sx: Annotated[Optional[str], pydantic.Field(alias="SX")] = None

    ss: Annotated[Optional[str], pydantic.Field(alias="SS")] = None

    xk: Annotated[Optional[str], pydantic.Field(alias="XK")] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
from dub.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class DeleteTagRequestTypedDict(TypedDict):
    id: str
    r"""The ID of the tag to delete."""


class DeleteTagRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The ID of the tag to delete."""


class DeleteTagResponseBodyTypedDict(TypedDict):
    r"""The deleted tag ID."""

    id: str
    r"""The ID of the deleted tag."""


class DeleteTagResponseBody(BaseModel):
    r"""The deleted tag ID."""

    id: str
    r"""The ID of the deleted tag."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .leadcreatedevent import LeadCreatedEvent, LeadCreatedEventTypedDict
from .linkclickedevent import LinkClickedEvent, LinkClickedEventTypedDict
from .linkwebhookevent import LinkWebhookEvent, LinkWebhookEventTypedDict
from .salecreatedevent import SaleCreatedEvent, SaleCreatedEventTypedDict
from typing import Union


WebhookEventTypedDict = Union[
    LinkWebhookEventTypedDict,
    LinkClickedEventTypedDict,
    LeadCreatedEventTypedDict,
    SaleCreatedEventTypedDict,
]
r"""Webhook event schema"""


WebhookEvent = Union[
    LinkWebhookEvent, LinkClickedEvent, LeadCreatedEvent, SaleCreatedEvent
]
r"""Webhook event schema"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict
from typing_extensions import Annotated, NotRequired, TypedDict


class TrackLeadRequestBodyTypedDict(TypedDict):
    click_id: str
    r"""The ID of the click in th Dub. You can read this value from `dub_id` cookie."""
    event_name: str
    r"""The name of the event to track."""
    customer_id: str
    r"""This is the unique identifier for the customer in the client's app. This is used to track the customer's journey."""
    customer_name: NotRequired[Nullable[str]]
    r"""Name of the customer in the client's app."""
    customer_email: NotRequired[Nullable[str]]
    r"""Email of the customer in the client's app."""
    customer_avatar: NotRequired[Nullable[str]]
    r"""Avatar of the customer in the client's app."""
    metadata: NotRequired[Nullable[Dict[str, Any]]]
    r"""Additional metadata to be stored with the lead event"""


class TrackLeadRequestBody(BaseModel):
    click_id: Annotated[str, pydantic.Field(alias="clickId")]
    r"""The ID of the click in th Dub. You can read this value from `dub_id` cookie."""

    event_name: Annotated[str, pydantic.Field(alias="eventName")]
    r"""The name of the event to track."""

    customer_id: Annotated[str, pydantic.Field(alias="customerId")]
    r"""This is the unique identifier for the customer in the client's app. This is used to track the customer's journey."""

    customer_name: Annotated[
        OptionalNullable[str], pydantic.Field(alias="customerName")
    ] = None
    r"""Name of the customer in the client's app."""

    customer_email: Annotated[
        OptionalNullable[str], pydantic.Field(alias="customerEmail")
    ] = None
    r"""Email of the customer in the client's app."""

    customer_avatar: Annotated[
        OptionalNullable[str], pydantic.Field(alias="customerAvatar")
    ] = None
    r"""Avatar of the customer in the client's app."""

    metadata: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Additional metadata to be stored with the lead event"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "customerName",
            "customerEmail",
            "customerAvatar",
            "metadata",
        ]
        nullable_fields = [
            "customerName",
            "customerEmail",
            "customerAvatar",
            "metadata",
        ]
        null_default_fields = ["customerName", "customerEmail", "customerAvatar"]

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ClickTypedDict(TypedDict):
    id: str


class Click(BaseModel):
    id: str


class CustomerTypedDict(TypedDict):
    id: str
    name: Nullable[str]
    email: Nullable[str]
    avatar: Nullable[str]


class Customer(BaseModel):
    id: str

    name: Nullable[str]

    email: Nullable[str]

    avatar: Nullable[str]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["name", "email", "avatar"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TrackLeadResponseBodyTypedDict(TypedDict):
    r"""A lead was tracked."""

    click: ClickTypedDict
    customer: CustomerTypedDict


class TrackLeadResponseBody(BaseModel):
    r"""A lead was tracked."""

    click: Click

    customer: Customer



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub import utils
from dub.types import BaseModel
from enum import Enum
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class UnauthorizedCode(str, Enum):
    r"""A short code indicating the error code returned."""

    UNAUTHORIZED = "unauthorized"


class UnauthorizedErrorTypedDict(TypedDict):
    code: UnauthorizedCode
    r"""A short code indicating the error code returned."""
    message: str
    r"""A human readable explanation of what went wrong."""
    doc_url: NotRequired[str]
    r"""A link to our documentation with more details about this error code"""


class UnauthorizedError(BaseModel):
    code: UnauthorizedCode
    r"""A short code indicating the error code returned."""

    message: str
    r"""A human readable explanation of what went wrong."""

    doc_url: Optional[str] = None
    r"""A link to our documentation with more details about this error code"""


class UnauthorizedData(BaseModel):
    error: UnauthorizedError


class Unauthorized(Exception):
    r"""Although the HTTP standard specifies \"unauthorized\", semantically this response means \"unauthenticated\". That is, the client must authenticate itself to get the requested response."""

    data: UnauthorizedData

    def __init__(self, data: UnauthorizedData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, UnauthorizedData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .badrequest import BadRequest, BadRequestData, Code, Error, ErrorTypedDict
from .conflict import (
    Conflict,
    ConflictCode,
    ConflictData,
    ConflictError,
    ConflictErrorTypedDict,
)
from .forbidden import (
    Forbidden,
    ForbiddenCode,
    ForbiddenData,
    ForbiddenError,
    ForbiddenErrorTypedDict,
)
from .internalservererror import (
    InternalServerError,
    InternalServerErrorCode,
    InternalServerErrorData,
    InternalServerErrorError,
    InternalServerErrorErrorTypedDict,
)
from .inviteexpired import (
    InviteExpired,
    InviteExpiredCode,
    InviteExpiredData,
    InviteExpiredError,
    InviteExpiredErrorTypedDict,
)
from .notfound import (
    NotFound,
    NotFoundCode,
    NotFoundData,
    NotFoundError,
    NotFoundErrorTypedDict,
)
from .ratelimitexceeded import (
    RateLimitExceeded,
    RateLimitExceededCode,
    RateLimitExceededData,
    RateLimitExceededError,
    RateLimitExceededErrorTypedDict,
)
from .sdkerror import SDKError
from .unauthorized import (
    Unauthorized,
    UnauthorizedCode,
    UnauthorizedData,
    UnauthorizedError,
    UnauthorizedErrorTypedDict,
)
from .unprocessableentity import (
    UnprocessableEntity,
    UnprocessableEntityCode,
    UnprocessableEntityData,
    UnprocessableEntityError,
    UnprocessableEntityErrorTypedDict,
)

__all__ = [
    "BadRequest",
    "BadRequestData",
    "Code",
    "Conflict",
    "ConflictCode",
    "ConflictData",
    "ConflictError",
    "ConflictErrorTypedDict",
    "Error",
    "ErrorTypedDict",
    "Forbidden",
    "ForbiddenCode",
    "ForbiddenData",
    "ForbiddenError",
    "ForbiddenErrorTypedDict",
    "InternalServerError",
    "InternalServerErrorCode",
    "InternalServerErrorData",
    "InternalServerErrorError",
    "InternalServerErrorErrorTypedDict",
    "InviteExpired",
    "InviteExpiredCode",
    "InviteExpiredData",
    "InviteExpiredError",
    "InviteExpiredErrorTypedDict",
    "NotFound",
    "NotFoundCode",
    "NotFoundData",
    "NotFoundError",
    "NotFoundErrorTypedDict",
    "RateLimitExceeded",
    "RateLimitExceededCode",
    "RateLimitExceededData",
    "RateLimitExceededError",
    "RateLimitExceededErrorTypedDict",
    "SDKError",
    "Unauthorized",
    "UnauthorizedCode",
    "UnauthorizedData",
    "UnauthorizedError",
    "UnauthorizedErrorTypedDict",
    "UnprocessableEntity",
    "UnprocessableEntityCode",
    "UnprocessableEntityData",
    "UnprocessableEntityError",
    "UnprocessableEntityErrorTypedDict",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel, Nullable, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class RegisteredDomainTypedDict(TypedDict):
    r"""The registered domain record."""

    id: str
    r"""The ID of the registered domain record."""
    created_at: str
    r"""The date the domain was created."""
    expires_at: str
    r"""The date the domain expires."""


class RegisteredDomain(BaseModel):
    r"""The registered domain record."""

    id: str
    r"""The ID of the registered domain record."""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The date the domain was created."""

    expires_at: Annotated[str, pydantic.Field(alias="expiresAt")]
    r"""The date the domain expires."""


class DomainSchemaTypedDict(TypedDict):
    id: str
    r"""The unique identifier of the domain."""
    slug: str
    r"""The domain name."""
    placeholder: Nullable[str]
    r"""Provide context to your teammates in the link creation modal by showing them an example of a link to be shortened."""
    expired_url: Nullable[str]
    r"""The URL to redirect to when a link under this domain has expired."""
    not_found_url: Nullable[str]
    r"""The URL to redirect to when a link under this domain doesn't exist."""
    created_at: str
    r"""The date the domain was created."""
    updated_at: str
    r"""The date the domain was last updated."""
    registered_domain: Nullable[RegisteredDomainTypedDict]
    r"""The registered domain record."""
    verified: NotRequired[bool]
    r"""Whether the domain is verified."""
    primary: NotRequired[bool]
    r"""Whether the domain is the primary domain for the workspace."""
    archived: NotRequired[bool]
    r"""Whether the domain is archived."""


class DomainSchema(BaseModel):
    id: str
    r"""The unique identifier of the domain."""

    slug: str
    r"""The domain name."""

    placeholder: Nullable[str]
    r"""Provide context to your teammates in the link creation modal by showing them an example of a link to be shortened."""

    expired_url: Annotated[Nullable[str], pydantic.Field(alias="expiredUrl")]
    r"""The URL to redirect to when a link under this domain has expired."""

    not_found_url: Annotated[Nullable[str], pydantic.Field(alias="notFoundUrl")]
    r"""The URL to redirect to when a link under this domain doesn't exist."""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The date the domain was created."""

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]
    r"""The date the domain was last updated."""

    registered_domain: Annotated[
        Nullable[RegisteredDomain], pydantic.Field(alias="registeredDomain")
    ]
    r"""The registered domain record."""

    verified: Optional[bool] = False
    r"""Whether the domain is verified."""

    primary: Optional[bool] = False
    r"""Whether the domain is the primary domain for the workspace."""

    archived: Optional[bool] = False
    r"""Whether the domain is archived."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["verified", "primary", "archived"]
        nullable_fields = [
            "placeholder",
            "expiredUrl",
            "notFoundUrl",
            "registeredDomain",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from dub import utils
from dub._hooks import HookContext
from dub.models import errors, operations
from dub.types import BaseModel, OptionalNullable, UNSET
from typing import Any, Optional, Union, cast


class QRCodes(BaseSDK):
    def get(
        self,
        *,
        request: Union[
            operations.GetQRCodeRequest, operations.GetQRCodeRequestTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[str]:
        r"""Retrieve a QR code

        Retrieve a QR code for a link.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.GetQRCodeRequest)
        request = cast(operations.GetQRCodeRequest, request)

        req = self.build_request(
            method="GET",
            path="/qr",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="image/png",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="getQRCode",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "410",
                "422",
                "429",
                "4XX",
                "500",
                "5XX",
            ],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "image/png"):
            return http_res.text
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.BadRequestData)
            raise errors.BadRequest(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.UnauthorizedData)
            raise errors.Unauthorized(data=data)
        if utils.match_response(http_res, "403", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.ForbiddenData)
            raise errors.Forbidden(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.NotFoundData)
            raise errors.NotFound(data=data)
        if utils.match_response(http_res, "409", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.ConflictData)
            raise errors.Conflict(data=data)
        if utils.match_response(http_res, "410", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.InviteExpiredData)
            raise errors.InviteExpired(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.UnprocessableEntityData)
            raise errors.UnprocessableEntity(data=data)
        if utils.match_response(http_res, "429", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.RateLimitExceededData)
            raise errors.RateLimitExceeded(data=data)
        if utils.match_response(http_res, "500", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.InternalServerErrorData)
            raise errors.InternalServerError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        request: Union[
            operations.GetQRCodeRequest, operations.GetQRCodeRequestTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> Optional[str]:
        r"""Retrieve a QR code

        Retrieve a QR code for a link.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, operations.GetQRCodeRequest)
        request = cast(operations.GetQRCodeRequest, request)

        req = self.build_request_async(
            method="GET",
            path="/qr",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="image/png",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="getQRCode",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "410",
                "422",
                "429",
                "4XX",
                "500",
                "5XX",
            ],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "image/png"):
            return http_res.text
        if utils.match_response(http_res, "400", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.BadRequestData)
            raise errors.BadRequest(data=data)
        if utils.match_response(http_res, "401", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.UnauthorizedData)
            raise errors.Unauthorized(data=data)
        if utils.match_response(http_res, "403", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.ForbiddenData)
            raise errors.Forbidden(data=data)
        if utils.match_response(http_res, "404", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.NotFoundData)
            raise errors.NotFound(data=data)
        if utils.match_response(http_res, "409", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.ConflictData)
            raise errors.Conflict(data=data)
        if utils.match_response(http_res, "410", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.InviteExpiredData)
            raise errors.InviteExpired(data=data)
        if utils.match_response(http_res, "422", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.UnprocessableEntityData)
            raise errors.UnprocessableEntity(data=data)
        if utils.match_response(http_res, "429", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.RateLimitExceededData)
            raise errors.RateLimitExceeded(data=data)
        if utils.match_response(http_res, "500", "application/json"):
            data = utils.unmarshal_json(http_res.text, errors.InternalServerErrorData)
            raise errors.InternalServerError(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
from enum import Enum
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Continent(str, Enum):
    r"""The 2-letter ISO 3166-1 code representing the continent associated with the location of the user."""

    AF = "AF"
    AN = "AN"
    AS = "AS"
    EU = "EU"
    NA = "NA"
    OC = "OC"
    SA = "SA"


class AnalyticsContinentsTypedDict(TypedDict):
    continent: Continent
    r"""The 2-letter ISO 3166-1 code representing the continent associated with the location of the user."""
    clicks: NotRequired[float]
    r"""The number of clicks from this continent"""
    leads: NotRequired[float]
    r"""The number of leads from this continent"""
    sales: NotRequired[float]
    r"""The number of sales from this continent"""
    sale_amount: NotRequired[float]
    r"""The total amount of sales from this continent, in cents"""


class AnalyticsContinents(BaseModel):
    continent: Continent
    r"""The 2-letter ISO 3166-1 code representing the continent associated with the location of the user."""

    clicks: Optional[float] = 0
    r"""The number of clicks from this continent"""

    leads: Optional[float] = 0
    r"""The number of leads from this continent"""

    sales: Optional[float] = 0
    r"""The number of sales from this continent"""

    sale_amount: Annotated[Optional[float], pydantic.Field(alias="saleAmount")] = 0
    r"""The total amount of sales from this continent, in cents"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateDomainRequestBodyTypedDict(TypedDict):
    slug: str
    r"""Name of the domain."""
    expired_url: NotRequired[Nullable[str]]
    r"""Redirect users to a specific URL when any link under this domain has expired."""
    not_found_url: NotRequired[Nullable[str]]
    r"""Redirect users to a specific URL when a link under this domain doesn't exist."""
    archived: NotRequired[bool]
    r"""Whether to archive this domain. `false` will unarchive a previously archived domain."""
    placeholder: NotRequired[Nullable[str]]
    r"""Provide context to your teammates in the link creation modal by showing them an example of a link to be shortened."""


class CreateDomainRequestBody(BaseModel):
    slug: str
    r"""Name of the domain."""

    expired_url: Annotated[
        OptionalNullable[str], pydantic.Field(alias="expiredUrl")
    ] = UNSET
    r"""Redirect users to a specific URL when any link under this domain has expired."""

    not_found_url: Annotated[
        OptionalNullable[str], pydantic.Field(alias="notFoundUrl")
    ] = UNSET
    r"""Redirect users to a specific URL when a link under this domain doesn't exist."""

    archived: Optional[bool] = False
    r"""Whether to archive this domain. `false` will unarchive a previously archived domain."""

    placeholder: OptionalNullable[str] = UNSET
    r"""Provide context to your teammates in the link creation modal by showing them an example of a link to be shortened."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["expiredUrl", "notFoundUrl", "archived", "placeholder"]
        nullable_fields = ["expiredUrl", "notFoundUrl", "placeholder"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel, Nullable, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TrackCustomerRequestBodyTypedDict(TypedDict):
    customer_id: str
    r"""This is the unique identifier for the customer in the client's app. This is used to track the customer's journey."""
    customer_name: NotRequired[str]
    r"""Name of the customer in the client's app."""
    customer_email: NotRequired[str]
    r"""Email of the customer in the client's app."""
    customer_avatar: NotRequired[str]
    r"""Avatar of the customer in the client's app."""


class TrackCustomerRequestBody(BaseModel):
    customer_id: Annotated[str, pydantic.Field(alias="customerId")]
    r"""This is the unique identifier for the customer in the client's app. This is used to track the customer's journey."""

    customer_name: Annotated[Optional[str], pydantic.Field(alias="customerName")] = None
    r"""Name of the customer in the client's app."""

    customer_email: Annotated[Optional[str], pydantic.Field(alias="customerEmail")] = (
        None
    )
    r"""Email of the customer in the client's app."""

    customer_avatar: Annotated[
        Optional[str], pydantic.Field(alias="customerAvatar")
    ] = None
    r"""Avatar of the customer in the client's app."""


class TrackCustomerResponseBodyTypedDict(TypedDict):
    r"""A customer was tracked."""

    customer_id: str
    customer_name: Nullable[str]
    customer_email: Nullable[str]
    customer_avatar: Nullable[str]


class TrackCustomerResponseBody(BaseModel):
    r"""A customer was tracked."""

    customer_id: Annotated[str, pydantic.Field(alias="customerId")]

    customer_name: Annotated[Nullable[str], pydantic.Field(alias="customerName")]

    customer_email: Annotated[Nullable[str], pydantic.Field(alias="customerEmail")]

    customer_avatar: Annotated[Nullable[str], pydantic.Field(alias="customerAvatar")]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["customerName", "customerEmail", "customerAvatar"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .analyticsbrowsers import AnalyticsBrowsers, AnalyticsBrowsersTypedDict
from .analyticscities import (
    AnalyticsCities,
    AnalyticsCitiesCountry,
    AnalyticsCitiesTypedDict,
)
from .analyticscontinents import (
    AnalyticsContinents,
    AnalyticsContinentsTypedDict,
    Continent,
)
from .analyticscount import AnalyticsCount, AnalyticsCountTypedDict
from .analyticscountries import (
    AnalyticsCountries,
    AnalyticsCountriesTypedDict,
    City,
    Country,
)
from .analyticsdevices import AnalyticsDevices, AnalyticsDevicesTypedDict
from .analyticsos import AnalyticsOS, AnalyticsOSTypedDict
from .analyticsreferers import AnalyticsReferers, AnalyticsReferersTypedDict
from .analyticsrefererurls import AnalyticsRefererUrls, AnalyticsRefererUrlsTypedDict
from .analyticstimeseries import AnalyticsTimeseries, AnalyticsTimeseriesTypedDict
from .analyticstoplinks import AnalyticsTopLinks, AnalyticsTopLinksTypedDict
from .analyticstopurls import AnalyticsTopUrls, AnalyticsTopUrlsTypedDict
from .analyticstriggers import AnalyticsTriggers, AnalyticsTriggersTypedDict, Trigger
from .clickevent import (
    Click,
    ClickEvent,
    ClickEventGeo,
    ClickEventGeoTypedDict,
    ClickEventTypedDict,
    ClickTypedDict,
    Event,
    Link,
    LinkTypedDict,
)
from .continentcode import ContinentCode
from .countrycode import CountryCode
from .domainschema import (
    DomainSchema,
    DomainSchemaTypedDict,
    RegisteredDomain,
    RegisteredDomainTypedDict,
)
from .leadcreatedevent import (
    LeadCreatedEvent,
    LeadCreatedEventClick,
    LeadCreatedEventClickTypedDict,
    LeadCreatedEventCustomer,
    LeadCreatedEventCustomerTypedDict,
    LeadCreatedEventData,
    LeadCreatedEventDataTypedDict,
    LeadCreatedEventEvent,
    LeadCreatedEventGeo,
    LeadCreatedEventGeoTypedDict,
    LeadCreatedEventLink,
    LeadCreatedEventLinkTypedDict,
    LeadCreatedEventTypedDict,
)
from .leadevent import (
    Customer,
    CustomerTypedDict,
    LeadEvent,
    LeadEventClick,
    LeadEventClickTypedDict,
    LeadEventEvent,
    LeadEventGeo,
    LeadEventGeoTypedDict,
    LeadEventLink,
    LeadEventLinkTypedDict,
    LeadEventTypedDict,
)
from .linkclickedevent import (
    LinkClickedEvent,
    LinkClickedEventClick,
    LinkClickedEventClickTypedDict,
    LinkClickedEventData,
    LinkClickedEventDataTypedDict,
    LinkClickedEventEvent,
    LinkClickedEventGeo,
    LinkClickedEventGeoTypedDict,
    LinkClickedEventLink,
    LinkClickedEventLinkTypedDict,
    LinkClickedEventTypedDict,
)
from .linkgeotargeting import LinkGeoTargeting, LinkGeoTargetingTypedDict
from .linkschema import Geo, GeoTypedDict, LinkSchema, LinkSchemaTypedDict
from .linkwebhookevent import (
    Data,
    DataTypedDict,
    LinkWebhookEvent,
    LinkWebhookEventEvent,
    LinkWebhookEventEventTypedDict,
    LinkWebhookEventGeo,
    LinkWebhookEventGeoTypedDict,
    LinkWebhookEventTypedDict,
    One,
    Three,
    Two,
)
from .salecreatedevent import (
    SaleCreatedEvent,
    SaleCreatedEventClick,
    SaleCreatedEventClickTypedDict,
    SaleCreatedEventCustomer,
    SaleCreatedEventCustomerTypedDict,
    SaleCreatedEventData,
    SaleCreatedEventDataTypedDict,
    SaleCreatedEventEvent,
    SaleCreatedEventGeo,
    SaleCreatedEventGeoTypedDict,
    SaleCreatedEventLink,
    SaleCreatedEventLinkTypedDict,
    SaleCreatedEventSale,
    SaleCreatedEventSaleTypedDict,
    SaleCreatedEventTypedDict,
)
from .saleevent import (
    PaymentProcessor,
    Sale,
    SaleEvent,
    SaleEventClick,
    SaleEventClickTypedDict,
    SaleEventCustomer,
    SaleEventCustomerTypedDict,
    SaleEventEvent,
    SaleEventGeo,
    SaleEventGeoTypedDict,
    SaleEventLink,
    SaleEventLinkTypedDict,
    SaleEventTypedDict,
    SaleTypedDict,
)
from .security import Security, SecurityTypedDict
from .tagschema import Color, TagSchema, TagSchemaTypedDict
from .webhookevent import WebhookEvent, WebhookEventTypedDict
from .workspaceschema import (
    Domains,
    DomainsTypedDict,
    Plan,
    Role,
    Users,
    UsersTypedDict,
    WorkspaceSchema,
    WorkspaceSchemaTypedDict,
)

__all__ = [
    "AnalyticsBrowsers",
    "AnalyticsBrowsersTypedDict",
    "AnalyticsCities",
    "AnalyticsCitiesCountry",
    "AnalyticsCitiesTypedDict",
    "AnalyticsContinents",
    "AnalyticsContinentsTypedDict",
    "AnalyticsCount",
    "AnalyticsCountTypedDict",
    "AnalyticsCountries",
    "AnalyticsCountriesTypedDict",
    "AnalyticsDevices",
    "AnalyticsDevicesTypedDict",
    "AnalyticsOS",
    "AnalyticsOSTypedDict",
    "AnalyticsRefererUrls",
    "AnalyticsRefererUrlsTypedDict",
    "AnalyticsReferers",
    "AnalyticsReferersTypedDict",
    "AnalyticsTimeseries",
    "AnalyticsTimeseriesTypedDict",
    "AnalyticsTopLinks",
    "AnalyticsTopLinksTypedDict",
    "AnalyticsTopUrls",
    "AnalyticsTopUrlsTypedDict",
    "AnalyticsTriggers",
    "AnalyticsTriggersTypedDict",
    "City",
    "Click",
    "ClickEvent",
    "ClickEventGeo",
    "ClickEventGeoTypedDict",
    "ClickEventTypedDict",
    "ClickTypedDict",
    "Color",
    "Continent",
    "ContinentCode",
    "Country",
    "CountryCode",
    "Customer",
    "CustomerTypedDict",
    "Data",
    "DataTypedDict",
    "DomainSchema",
    "DomainSchemaTypedDict",
    "Domains",
    "DomainsTypedDict",
    "Event",
    "Geo",
    "GeoTypedDict",
    "LeadCreatedEvent",
    "LeadCreatedEventClick",
    "LeadCreatedEventClickTypedDict",
    "LeadCreatedEventCustomer",
    "LeadCreatedEventCustomerTypedDict",
    "LeadCreatedEventData",
    "LeadCreatedEventDataTypedDict",
    "LeadCreatedEventEvent",
    "LeadCreatedEventGeo",
    "LeadCreatedEventGeoTypedDict",
    "LeadCreatedEventLink",
    "LeadCreatedEventLinkTypedDict",
    "LeadCreatedEventTypedDict",
    "LeadEvent",
    "LeadEventClick",
    "LeadEventClickTypedDict",
    "LeadEventEvent",
    "LeadEventGeo",
    "LeadEventGeoTypedDict",
    "LeadEventLink",
    "LeadEventLinkTypedDict",
    "LeadEventTypedDict",
    "Link",
    "LinkClickedEvent",
    "LinkClickedEventClick",
    "LinkClickedEventClickTypedDict",
    "LinkClickedEventData",
    "LinkClickedEventDataTypedDict",
    "LinkClickedEventEvent",
    "LinkClickedEventGeo",
    "LinkClickedEventGeoTypedDict",
    "LinkClickedEventLink",
    "LinkClickedEventLinkTypedDict",
    "LinkClickedEventTypedDict",
    "LinkGeoTargeting",
    "LinkGeoTargetingTypedDict",
    "LinkSchema",
    "LinkSchemaTypedDict",
    "LinkTypedDict",
    "LinkWebhookEvent",
    "LinkWebhookEventEvent",
    "LinkWebhookEventEventTypedDict",
    "LinkWebhookEventGeo",
    "LinkWebhookEventGeoTypedDict",
    "LinkWebhookEventTypedDict",
    "One",
    "PaymentProcessor",
    "Plan",
    "RegisteredDomain",
    "RegisteredDomainTypedDict",
    "Role",
    "Sale",
    "SaleCreatedEvent",
    "SaleCreatedEventClick",
    "SaleCreatedEventClickTypedDict",
    "SaleCreatedEventCustomer",
    "SaleCreatedEventCustomerTypedDict",
    "SaleCreatedEventData",
    "SaleCreatedEventDataTypedDict",
    "SaleCreatedEventEvent",
    "SaleCreatedEventGeo",
    "SaleCreatedEventGeoTypedDict",
    "SaleCreatedEventLink",
    "SaleCreatedEventLinkTypedDict",
    "SaleCreatedEventSale",
    "SaleCreatedEventSaleTypedDict",
    "SaleCreatedEventTypedDict",
    "SaleEvent",
    "SaleEventClick",
    "SaleEventClickTypedDict",
    "SaleEventCustomer",
    "SaleEventCustomerTypedDict",
    "SaleEventEvent",
    "SaleEventGeo",
    "SaleEventGeoTypedDict",
    "SaleEventLink",
    "SaleEventLinkTypedDict",
    "SaleEventTypedDict",
    "SaleTypedDict",
    "Security",
    "SecurityTypedDict",
    "TagSchema",
    "TagSchemaTypedDict",
    "Three",
    "Trigger",
    "Two",
    "Users",
    "UsersTypedDict",
    "WebhookEvent",
    "WebhookEventTypedDict",
    "WorkspaceSchema",
    "WorkspaceSchemaTypedDict",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
from dub.utils import FieldMetadata, QueryParamMetadata
import pydantic
from typing import List
from typing_extensions import Annotated, TypedDict


class BulkDeleteLinksRequestTypedDict(TypedDict):
    link_ids: List[str]
    r"""Comma-separated list of link IDs to delete. Maximum of 100 IDs. Non-existing IDs will be ignored."""


class BulkDeleteLinksRequest(BaseModel):
    link_ids: Annotated[
        List[str],
        pydantic.Field(alias="linkIds"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ]
    r"""Comma-separated list of link IDs to delete. Maximum of 100 IDs. Non-existing IDs will be ignored."""


class BulkDeleteLinksResponseBodyTypedDict(TypedDict):
    r"""The deleted links count."""

    deleted_count: float
    r"""The number of links deleted."""


class BulkDeleteLinksResponseBody(BaseModel):
    r"""The deleted links count."""

    deleted_count: Annotated[float, pydantic.Field(alias="deletedCount")]
    r"""The number of links deleted."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AnalyticsTopUrlsTypedDict(TypedDict):
    url: str
    r"""The destination URL"""
    clicks: NotRequired[float]
    r"""The number of clicks from this URL"""
    leads: NotRequired[float]
    r"""The number of leads from this URL"""
    sales: NotRequired[float]
    r"""The number of sales from this URL"""
    sale_amount: NotRequired[float]
    r"""The total amount of sales from this URL, in cents"""


class AnalyticsTopUrls(BaseModel):
    url: str
    r"""The destination URL"""

    clicks: Optional[float] = 0
    r"""The number of clicks from this URL"""

    leads: Optional[float] = 0
    r"""The number of leads from this URL"""

    sales: Optional[float] = 0
    r"""The number of sales from this URL"""

    sale_amount: Annotated[Optional[float], pydantic.Field(alias="saleAmount")] = 0
    r"""The total amount of sales from this URL, in cents"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
from dub.utils import FieldMetadata, QueryParamMetadata
from enum import Enum
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Level(str, Enum):
    r"""The level of error correction to use for the QR code. Defaults to `L` if not provided."""

    L = "L"
    M = "M"
    Q = "Q"
    H = "H"


class GetQRCodeRequestTypedDict(TypedDict):
    url: str
    r"""The URL to generate a QR code for."""
    logo: NotRequired[str]
    r"""The logo to include in the QR code. Can only be used with a paid plan on Dub.co."""
    size: NotRequired[float]
    r"""The size of the QR code in pixels. Defaults to `600` if not provided."""
    level: NotRequired[Level]
    r"""The level of error correction to use for the QR code. Defaults to `L` if not provided."""
    fg_color: NotRequired[str]
    r"""The foreground color of the QR code in hex format. Defaults to `#000000` if not provided."""
    bg_color: NotRequired[str]
    r"""The background color of the QR code in hex format. Defaults to `#ffffff` if not provided."""
    hide_logo: NotRequired[bool]
    r"""Whether to hide the logo in the QR code. Can only be used with a paid plan on Dub.co."""
    include_margin: NotRequired[bool]
    r"""Whether to include a margin around the QR code. Defaults to `false` if not provided."""


class GetQRCodeRequest(BaseModel):
    url: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]
    r"""The URL to generate a QR code for."""

    logo: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The logo to include in the QR code. Can only be used with a paid plan on Dub.co."""

    size: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 600
    r"""The size of the QR code in pixels. Defaults to `600` if not provided."""

    level: Annotated[
        Optional[Level],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = Level.L
    r"""The level of error correction to use for the QR code. Defaults to `L` if not provided."""

    fg_color: Annotated[
        Optional[str],
        pydantic.Field(alias="fgColor"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = "#000000"
    r"""The foreground color of the QR code in hex format. Defaults to `#000000` if not provided."""

    bg_color: Annotated[
        Optional[str],
        pydantic.Field(alias="bgColor"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = "#FFFFFF"
    r"""The background color of the QR code in hex format. Defaults to `#ffffff` if not provided."""

    hide_logo: Annotated[
        Optional[bool],
        pydantic.Field(alias="hideLogo"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = True
    r"""Whether to hide the logo in the QR code. Can only be used with a paid plan on Dub.co."""

    include_margin: Annotated[
        Optional[bool],
        pydantic.Field(alias="includeMargin"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = True
    r"""Whether to include a margin around the QR code. Defaults to `false` if not provided."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ContinentCode(str, Enum):
    r"""The continent to retrieve analytics for."""

    AF = "AF"
    AN = "AN"
    AS = "AS"
    EU = "EU"
    NA = "NA"
    OC = "OC"
    SA = "SA"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
from dub.utils import FieldMetadata, QueryParamMetadata
from enum import Enum
import pydantic
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


GetLinksCountQueryParamTagIdsTypedDict = Union[str, List[str]]
r"""The tag IDs to filter the links by."""


GetLinksCountQueryParamTagIds = Union[str, List[str]]
r"""The tag IDs to filter the links by."""


GetLinksCountQueryParamTagNamesTypedDict = Union[str, List[str]]
r"""The unique name of the tags assigned to the short link (case insensitive)."""


GetLinksCountQueryParamTagNames = Union[str, List[str]]
r"""The unique name of the tags assigned to the short link (case insensitive)."""


class Three(str, Enum):
    USER_ID = "userId"


class Two(str, Enum):
    TAG_ID = "tagId"


class One(str, Enum):
    DOMAIN = "domain"


GroupByTypedDict = Union[One, Two, Three]
r"""The field to group the links by."""


GroupBy = Union[One, Two, Three]
r"""The field to group the links by."""


class GetLinksCountRequestTypedDict(TypedDict):
    domain: NotRequired[str]
    r"""The domain to filter the links by. E.g. `ac.me`. If not provided, all links for the workspace will be returned."""
    tag_id: NotRequired[str]
    r"""The tag ID to filter the links by. This field is deprecated – use `tagIds` instead."""
    tag_ids: NotRequired[GetLinksCountQueryParamTagIdsTypedDict]
    r"""The tag IDs to filter the links by."""
    tag_names: NotRequired[GetLinksCountQueryParamTagNamesTypedDict]
    r"""The unique name of the tags assigned to the short link (case insensitive)."""
    search: NotRequired[str]
    r"""The search term to filter the links by. The search term will be matched against the short link slug and the destination url."""
    user_id: NotRequired[str]
    r"""The user ID to filter the links by."""
    show_archived: NotRequired[bool]
    r"""Whether to include archived links in the response. Defaults to `false` if not provided."""
    with_tags: NotRequired[bool]
    r"""DEPRECATED. Filter for links that have at least one tag assigned to them."""
    group_by: NotRequired[GroupByTypedDict]
    r"""The field to group the links by."""


class GetLinksCountRequest(BaseModel):
    domain: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The domain to filter the links by. E.g. `ac.me`. If not provided, all links for the workspace will be returned."""

    tag_id: Annotated[
        Optional[str],
        pydantic.Field(alias="tagId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The tag ID to filter the links by. This field is deprecated – use `tagIds` instead."""

    tag_ids: Annotated[
        Optional[GetLinksCountQueryParamTagIds],
        pydantic.Field(alias="tagIds"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The tag IDs to filter the links by."""

    tag_names: Annotated[
        Optional[GetLinksCountQueryParamTagNames],
        pydantic.Field(alias="tagNames"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The unique name of the tags assigned to the short link (case insensitive)."""

    search: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The search term to filter the links by. The search term will be matched against the short link slug and the destination url."""

    user_id: Annotated[
        Optional[str],
        pydantic.Field(alias="userId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The user ID to filter the links by."""

    show_archived: Annotated[
        Optional[bool],
        pydantic.Field(alias="showArchived"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = True
    r"""Whether to include archived links in the response. Defaults to `false` if not provided."""

    with_tags: Annotated[
        Optional[bool],
        pydantic.Field(alias="withTags"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = True
    r"""DEPRECATED. Filter for links that have at least one tag assigned to them."""

    group_by: Annotated[
        Optional[GroupBy],
        pydantic.Field(alias="groupBy"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The field to group the links by."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub import utils
from dub.types import BaseModel
from enum import Enum
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class Code(str, Enum):
    r"""A short code indicating the error code returned."""

    BAD_REQUEST = "bad_request"


class ErrorTypedDict(TypedDict):
    code: Code
    r"""A short code indicating the error code returned."""
    message: str
    r"""A human readable explanation of what went wrong."""
    doc_url: NotRequired[str]
    r"""A link to our documentation with more details about this error code"""


class Error(BaseModel):
    code: Code
    r"""A short code indicating the error code returned."""

    message: str
    r"""A human readable explanation of what went wrong."""

    doc_url: Optional[str] = None
    r"""A link to our documentation with more details about this error code"""


class BadRequestData(BaseModel):
    error: Error


class BadRequest(Exception):
    r"""The server cannot or will not process the request due to something that is perceived to be a client error (e.g., malformed request syntax, invalid request message framing, or deceptive request routing)."""

    data: BadRequestData

    def __init__(self, data: BadRequestData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, BadRequestData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
from dub.utils import FieldMetadata, PathParamMetadata
import pydantic
from typing_extensions import Annotated, TypedDict


class GetWorkspaceRequestTypedDict(TypedDict):
    id_or_slug: str
    r"""The ID or slug of the workspace."""


class GetWorkspaceRequest(BaseModel):
    id_or_slug: Annotated[
        str,
        pydantic.Field(alias="idOrSlug"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""The ID or slug of the workspace."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.models.components import linkgeotargeting as components_linkgeotargeting
from dub.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


TagIdsTypedDict = Union[str, List[str]]
r"""The unique IDs of the tags assigned to the short link."""


TagIds = Union[str, List[str]]
r"""The unique IDs of the tags assigned to the short link."""


TagNamesTypedDict = Union[str, List[str]]
r"""The unique name of the tags assigned to the short link (case insensitive)."""


TagNames = Union[str, List[str]]
r"""The unique name of the tags assigned to the short link (case insensitive)."""


class CreateLinkRequestBodyTypedDict(TypedDict):
    url: str
    r"""The destination URL of the short link."""
    domain: NotRequired[str]
    r"""The domain of the short link. If not provided, the primary domain for the workspace will be used (or `dub.sh` if the workspace has no domains)."""
    key: NotRequired[str]
    r"""The short link slug. If not provided, a random 7-character slug will be generated."""
    external_id: NotRequired[Nullable[str]]
    r"""This is the ID of the link in your database. If set, it can be used to identify the link in the future. Must be prefixed with `ext_` when passed as a query parameter."""
    prefix: NotRequired[str]
    r"""The prefix of the short link slug for randomly-generated keys (e.g. if prefix is `/c/`, generated keys will be in the `/c/:key` format). Will be ignored if `key` is provided."""
    track_conversion: NotRequired[bool]
    r"""Whether to track conversions for the short link."""
    identifier: NotRequired[Nullable[str]]
    r"""The identifier of the short link that is unique across your workspace. If set, it can be used to identify your short link for client-side click tracking."""
    archived: NotRequired[bool]
    r"""Whether the short link is archived."""
    public_stats: NotRequired[bool]
    r"""Deprecated: Use `dashboard` instead. Whether the short link's stats are publicly accessible."""
    tag_id: NotRequired[Nullable[str]]
    r"""The unique ID of the tag assigned to the short link. This field is deprecated – use `tagIds` instead."""
    tag_ids: NotRequired[TagIdsTypedDict]
    r"""The unique IDs of the tags assigned to the short link."""
    tag_names: NotRequired[TagNamesTypedDict]
    r"""The unique name of the tags assigned to the short link (case insensitive)."""
    comments: NotRequired[Nullable[str]]
    r"""The comments for the short link."""
    expires_at: NotRequired[Nullable[str]]
    r"""The date and time when the short link will expire at."""
    expired_url: NotRequired[Nullable[str]]
    r"""The URL to redirect to when the short link has expired."""
    password: NotRequired[Nullable[str]]
    r"""The password required to access the destination URL of the short link."""
    proxy: NotRequired[bool]
    r"""Whether the short link uses Custom Social Media Cards feature."""
    title: NotRequired[Nullable[str]]
    r"""The custom link preview title (og:title). Will be used for Custom Social Media Cards if `proxy` is true. Learn more: https://d.to/og"""
    description: NotRequired[Nullable[str]]
    r"""The custom link preview description (og:description). Will be used for Custom Social Media Cards if `proxy` is true. Learn more: https://d.to/og"""
    image: NotRequired[Nullable[str]]
    r"""The custom link preview image (og:image). Will be used for Custom Social Media Cards if `proxy` is true. Learn more: https://d.to/og"""
    video: NotRequired[Nullable[str]]
    r"""The custom link preview video (og:video). Will be used for Custom Social Media Cards if `proxy` is true. Learn more: https://d.to/og"""
    rewrite: NotRequired[bool]
    r"""Whether the short link uses link cloaking."""
    ios: NotRequired[Nullable[str]]
    r"""The iOS destination URL for the short link for iOS device targeting."""
    android: NotRequired[Nullable[str]]
    r"""The Android destination URL for the short link for Android device targeting."""
    geo: NotRequired[Nullable[components_linkgeotargeting.LinkGeoTargetingTypedDict]]
    r"""Geo targeting information for the short link in JSON format `{[COUNTRY]: https://example.com }`."""
    do_index: NotRequired[bool]
    r"""Allow search engines to index your short link. Defaults to `false` if not provided. Learn more: https://d.to/noindex"""
    utm_source: NotRequired[Nullable[str]]
    r"""The UTM source of the short link. If set, this will populate or override the UTM source in the destination URL."""
    utm_medium: NotRequired[Nullable[str]]
    r"""The UTM medium of the short link. If set, this will populate or override the UTM medium in the destination URL."""
    utm_campaign: NotRequired[Nullable[str]]
    r"""The UTM campaign of the short link. If set, this will populate or override the UTM campaign in the destination URL."""
    utm_term: NotRequired[Nullable[str]]
    r"""The UTM term of the short link. If set, this will populate or override the UTM term in the destination URL."""
    utm_content: NotRequired[Nullable[str]]
    r"""The UTM content of the short link. If set, this will populate or override the UTM content in the destination URL."""
    ref: NotRequired[Nullable[str]]
    r"""The referral tag of the short link. If set, this will populate or override the `ref` query parameter in the destination URL."""
    webhook_ids: NotRequired[Nullable[List[str]]]
    r"""An array of webhook IDs to trigger when the link is clicked. These webhooks will receive click event data."""


class CreateLinkRequestBody(BaseModel):
    url: str
    r"""The destination URL of the short link."""

    domain: Optional[str] = None
    r"""The domain of the short link. If not provided, the primary domain for the workspace will be used (or `dub.sh` if the workspace has no domains)."""

    key: Optional[str] = None
    r"""The short link slug. If not provided, a random 7-character slug will be generated."""

    external_id: Annotated[
        OptionalNullable[str], pydantic.Field(alias="externalId")
    ] = UNSET
    r"""This is the ID of the link in your database. If set, it can be used to identify the link in the future. Must be prefixed with `ext_` when passed as a query parameter."""

    prefix: Optional[str] = None
    r"""The prefix of the short link slug for randomly-generated keys (e.g. if prefix is `/c/`, generated keys will be in the `/c/:key` format). Will be ignored if `key` is provided."""

    track_conversion: Annotated[
        Optional[bool], pydantic.Field(alias="trackConversion")
    ] = False
    r"""Whether to track conversions for the short link."""

    identifier: OptionalNullable[str] = UNSET
    r"""The identifier of the short link that is unique across your workspace. If set, it can be used to identify your short link for client-side click tracking."""

    archived: Optional[bool] = False
    r"""Whether the short link is archived."""

    public_stats: Annotated[
        Optional[bool],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="publicStats",
        ),
    ] = False
    r"""Deprecated: Use `dashboard` instead. Whether the short link's stats are publicly accessible."""

    tag_id: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible.",
            alias="tagId",
        ),
    ] = UNSET
    r"""The unique ID of the tag assigned to the short link. This field is deprecated – use `tagIds` instead."""

    tag_ids: Annotated[Optional[TagIds], pydantic.Field(alias="tagIds")] = None
    r"""The unique IDs of the tags assigned to the short link."""

    tag_names: Annotated[Optional[TagNames], pydantic.Field(alias="tagNames")] = None
    r"""The unique name of the tags assigned to the short link (case insensitive)."""

    comments: OptionalNullable[str] = UNSET
    r"""The comments for the short link."""

    expires_at: Annotated[OptionalNullable[str], pydantic.Field(alias="expiresAt")] = (
        UNSET
    )
    r"""The date and time when the short link will expire at."""

    expired_url: Annotated[
        OptionalNullable[str], pydantic.Field(alias="expiredUrl")
    ] = UNSET
    r"""The URL to redirect to when the short link has expired."""

    password: OptionalNullable[str] = UNSET
    r"""The password required to access the destination URL of the short link."""

    proxy: Optional[bool] = False
    r"""Whether the short link uses Custom Social Media Cards feature."""

    title: OptionalNullable[str] = UNSET
    r"""The custom link preview title (og:title). Will be used for Custom Social Media Cards if `proxy` is true. Learn more: https://d.to/og"""

    description: OptionalNullable[str] = UNSET
    r"""The custom link preview description (og:description). Will be used for Custom Social Media Cards if `proxy` is true. Learn more: https://d.to/og"""

    image: OptionalNullable[str] = UNSET
    r"""The custom link preview image (og:image). Will be used for Custom Social Media Cards if `proxy` is true. Learn more: https://d.to/og"""

    video: OptionalNullable[str] = UNSET
    r"""The custom link preview video (og:video). Will be used for Custom Social Media Cards if `proxy` is true. Learn more: https://d.to/og"""

    rewrite: Optional[bool] = False
    r"""Whether the short link uses link cloaking."""

    ios: OptionalNullable[str] = UNSET
    r"""The iOS destination URL for the short link for iOS device targeting."""

    android: OptionalNullable[str] = UNSET
    r"""The Android destination URL for the short link for Android device targeting."""

    geo: OptionalNullable[components_linkgeotargeting.LinkGeoTargeting] = UNSET
    r"""Geo targeting information for the short link in JSON format `{[COUNTRY]: https://example.com }`."""

    do_index: Annotated[Optional[bool], pydantic.Field(alias="doIndex")] = False
    r"""Allow search engines to index your short link. Defaults to `false` if not provided. Learn more: https://d.to/noindex"""

    utm_source: OptionalNullable[str] = UNSET
    r"""The UTM source of the short link. If set, this will populate or override the UTM source in the destination URL."""

    utm_medium: OptionalNullable[str] = UNSET
    r"""The UTM medium of the short link. If set, this will populate or override the UTM medium in the destination URL."""

    utm_campaign: OptionalNullable[str] = UNSET
    r"""The UTM campaign of the short link. If set, this will populate or override the UTM campaign in the destination URL."""

    utm_term: OptionalNullable[str] = UNSET
    r"""The UTM term of the short link. If set, this will populate or override the UTM term in the destination URL."""

    utm_content: OptionalNullable[str] = UNSET
    r"""The UTM content of the short link. If set, this will populate or override the UTM content in the destination URL."""

    ref: OptionalNullable[str] = UNSET
    r"""The referral tag of the short link. If set, this will populate or override the `ref` query parameter in the destination URL."""

    webhook_ids: Annotated[
        OptionalNullable[List[str]], pydantic.Field(alias="webhookIds")
    ] = UNSET
    r"""An array of webhook IDs to trigger when the link is clicked. These webhooks will receive click event data."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "domain",
            "key",
            "externalId",
            "prefix",
            "trackConversion",
            "identifier",
            "archived",
            "publicStats",
            "tagId",
            "tagIds",
            "tagNames",
            "comments",
            "expiresAt",
            "expiredUrl",
            "password",
            "proxy",
            "title",
            "description",
            "image",
            "video",
            "rewrite",
            "ios",
            "android",
            "geo",
            "doIndex",
            "utm_source",
            "utm_medium",
            "utm_campaign",
            "utm_term",
            "utm_content",
            "ref",
            "webhookIds",
        ]
        nullable_fields = [
            "externalId",
            "identifier",
            "tagId",
            "comments",
            "expiresAt",
            "expiredUrl",
            "password",
            "title",
            "description",
            "image",
            "video",
            "ios",
            "android",
            "geo",
            "utm_source",
            "utm_medium",
            "utm_campaign",
            "utm_term",
            "utm_content",
            "ref",
            "webhookIds",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
from dub.utils import FieldMetadata, PathParamMetadata, RequestMetadata
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UpdateWorkspaceRequestBodyTypedDict(TypedDict):
    name: NotRequired[str]
    slug: NotRequired[str]


class UpdateWorkspaceRequestBody(BaseModel):
    name: Optional[str] = None

    slug: Optional[str] = None


class UpdateWorkspaceRequestTypedDict(TypedDict):
    id_or_slug: str
    r"""The ID or slug of the workspace to update."""
    request_body: NotRequired[UpdateWorkspaceRequestBodyTypedDict]


class UpdateWorkspaceRequest(BaseModel):
    id_or_slug: Annotated[
        str,
        pydantic.Field(alias="idOrSlug"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""The ID or slug of the workspace to update."""

    request_body: Annotated[
        Optional[UpdateWorkspaceRequestBody],
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AnalyticsReferersTypedDict(TypedDict):
    referer: str
    r"""The name of the referer. If unknown, this will be `(direct)`"""
    clicks: NotRequired[float]
    r"""The number of clicks from this referer"""
    leads: NotRequired[float]
    r"""The number of leads from this referer"""
    sales: NotRequired[float]
    r"""The number of sales from this referer"""
    sale_amount: NotRequired[float]
    r"""The total amount of sales from this referer, in cents"""


class AnalyticsReferers(BaseModel):
    referer: str
    r"""The name of the referer. If unknown, this will be `(direct)`"""

    clicks: Optional[float] = 0
    r"""The number of clicks from this referer"""

    leads: Optional[float] = 0
    r"""The number of leads from this referer"""

    sales: Optional[float] = 0
    r"""The number of sales from this referer"""

    sale_amount: Annotated[Optional[float], pydantic.Field(alias="saleAmount")] = 0
    r"""The total amount of sales from this referer, in cents"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AnalyticsTopLinksTypedDict(TypedDict):
    link: str
    r"""The unique ID of the short link"""
    id: str
    r"""The unique ID of the short link"""
    domain: str
    r"""The domain of the short link"""
    key: str
    r"""The key of the short link"""
    short_link: str
    r"""The short link URL"""
    url: str
    r"""The destination URL of the short link"""
    created_at: str
    r"""The creation timestamp of the short link"""
    clicks: NotRequired[float]
    r"""The number of clicks from this link"""
    leads: NotRequired[float]
    r"""The number of leads from this link"""
    sales: NotRequired[float]
    r"""The number of sales from this link"""
    sale_amount: NotRequired[float]
    r"""The total amount of sales from this link, in cents"""


class AnalyticsTopLinks(BaseModel):
    link: Annotated[
        str,
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]
    r"""The unique ID of the short link"""

    id: str
    r"""The unique ID of the short link"""

    domain: str
    r"""The domain of the short link"""

    key: str
    r"""The key of the short link"""

    short_link: Annotated[str, pydantic.Field(alias="shortLink")]
    r"""The short link URL"""

    url: str
    r"""The destination URL of the short link"""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The creation timestamp of the short link"""

    clicks: Optional[float] = 0
    r"""The number of clicks from this link"""

    leads: Optional[float] = 0
    r"""The number of leads from this link"""

    sales: Optional[float] = 0
    r"""The number of sales from this link"""

    sale_amount: Annotated[Optional[float], pydantic.Field(alias="saleAmount")] = 0
    r"""The total amount of sales from this link, in cents"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
from dub.utils import FieldMetadata, PathParamMetadata
import pydantic
from typing_extensions import Annotated, TypedDict


class DeleteLinkRequestTypedDict(TypedDict):
    link_id: str
    r"""The id of the link to delete. You may use either `linkId` (obtained via `/links/info` endpoint) or `externalId` prefixed with `ext_`."""


class DeleteLinkRequest(BaseModel):
    link_id: Annotated[
        str,
        pydantic.Field(alias="linkId"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]
    r"""The id of the link to delete. You may use either `linkId` (obtained via `/links/info` endpoint) or `externalId` prefixed with `ext_`."""


class DeleteLinkResponseBodyTypedDict(TypedDict):
    r"""The deleted link ID."""

    id: str
    r"""The ID of the link."""


class DeleteLinkResponseBody(BaseModel):
    r"""The deleted link ID."""

    id: str
    r"""The ID of the link."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from enum import Enum
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PaymentProcessor(str, Enum):
    r"""The payment processor via which the sale was made."""

    STRIPE = "stripe"
    SHOPIFY = "shopify"
    PADDLE = "paddle"


class TrackSaleRequestBodyTypedDict(TypedDict):
    customer_id: str
    r"""This is the unique identifier for the customer in the client's app. This is used to track the customer's journey."""
    amount: int
    r"""The amount of the sale. Should be passed in cents."""
    payment_processor: PaymentProcessor
    r"""The payment processor via which the sale was made."""
    event_name: NotRequired[str]
    r"""The name of the sale event. It can be used to track different types of event for example 'Purchase', 'Upgrade', 'Payment', etc."""
    invoice_id: NotRequired[Nullable[str]]
    r"""The invoice ID of the sale."""
    currency: NotRequired[str]
    r"""The currency of the sale. Accepts ISO 4217 currency codes."""
    metadata: NotRequired[Nullable[Dict[str, Any]]]
    r"""Additional metadata to be stored with the sale event."""


class TrackSaleRequestBody(BaseModel):
    customer_id: Annotated[str, pydantic.Field(alias="customerId")]
    r"""This is the unique identifier for the customer in the client's app. This is used to track the customer's journey."""

    amount: int
    r"""The amount of the sale. Should be passed in cents."""

    payment_processor: Annotated[
        PaymentProcessor, pydantic.Field(alias="paymentProcessor")
    ]
    r"""The payment processor via which the sale was made."""

    event_name: Annotated[Optional[str], pydantic.Field(alias="eventName")] = "Purchase"
    r"""The name of the sale event. It can be used to track different types of event for example 'Purchase', 'Upgrade', 'Payment', etc."""

    invoice_id: Annotated[OptionalNullable[str], pydantic.Field(alias="invoiceId")] = (
        None
    )
    r"""The invoice ID of the sale."""

    currency: Optional[str] = "usd"
    r"""The currency of the sale. Accepts ISO 4217 currency codes."""

    metadata: OptionalNullable[Dict[str, Any]] = UNSET
    r"""Additional metadata to be stored with the sale event."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["eventName", "invoiceId", "currency", "metadata"]
        nullable_fields = ["invoiceId", "metadata"]
        null_default_fields = ["invoiceId"]

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TrackSaleCustomerTypedDict(TypedDict):
    id: str
    name: Nullable[str]
    email: Nullable[str]
    avatar: Nullable[str]


class TrackSaleCustomer(BaseModel):
    id: str

    name: Nullable[str]

    email: Nullable[str]

    avatar: Nullable[str]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["name", "email", "avatar"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class SaleTypedDict(TypedDict):
    amount: float
    currency: str
    payment_processor: str
    invoice_id: Nullable[str]
    metadata: Nullable[Dict[str, Any]]


class Sale(BaseModel):
    amount: float

    currency: str

    payment_processor: Annotated[str, pydantic.Field(alias="paymentProcessor")]

    invoice_id: Annotated[Nullable[str], pydantic.Field(alias="invoiceId")]

    metadata: Nullable[Dict[str, Any]]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["invoiceId", "metadata"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TrackSaleResponseBodyTypedDict(TypedDict):
    r"""A sale was tracked."""

    event_name: str
    customer: TrackSaleCustomerTypedDict
    sale: SaleTypedDict


class TrackSaleResponseBody(BaseModel):
    r"""A sale was tracked."""

    event_name: Annotated[str, pydantic.Field(alias="eventName")]

    customer: TrackSaleCustomer

    sale: Sale



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
from enum import Enum
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Color(str, Enum):
    r"""The color of the tag. If not provided, a random color will be used from the list: red, yellow, green, blue, purple, pink, brown."""

    RED = "red"
    YELLOW = "yellow"
    GREEN = "green"
    BLUE = "blue"
    PURPLE = "purple"
    PINK = "pink"
    BROWN = "brown"


class CreateTagRequestBodyTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of the tag to create."""
    color: NotRequired[Color]
    r"""The color of the tag. If not provided, a random color will be used from the list: red, yellow, green, blue, purple, pink, brown."""
    tag: NotRequired[str]
    r"""The name of the tag to create."""


class CreateTagRequestBody(BaseModel):
    name: Optional[str] = None
    r"""The name of the tag to create."""

    color: Optional[Color] = None
    r"""The color of the tag. If not provided, a random color will be used from the list: red, yellow, green, blue, purple, pink, brown."""

    tag: Annotated[
        Optional[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = None
    r"""The name of the tag to create."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.models.components import linkschema as components_linkschema
from dub.types import BaseModel
from dub.utils import FieldMetadata, QueryParamMetadata
from enum import Enum
import pydantic
from typing import Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


QueryParamTagIdsTypedDict = Union[str, List[str]]
r"""The tag IDs to filter the links by."""


QueryParamTagIds = Union[str, List[str]]
r"""The tag IDs to filter the links by."""


QueryParamTagNamesTypedDict = Union[str, List[str]]
r"""The unique name of the tags assigned to the short link (case insensitive)."""


QueryParamTagNames = Union[str, List[str]]
r"""The unique name of the tags assigned to the short link (case insensitive)."""


class Sort(str, Enum):
    r"""The field to sort the links by. The default is `createdAt`, and sort order is always descending."""

    CREATED_AT = "createdAt"
    CLICKS = "clicks"
    LAST_CLICKED = "lastClicked"


class GetLinksRequestTypedDict(TypedDict):
    domain: NotRequired[str]
    r"""The domain to filter the links by. E.g. `ac.me`. If not provided, all links for the workspace will be returned."""
    tag_id: NotRequired[str]
    r"""The tag ID to filter the links by. This field is deprecated – use `tagIds` instead."""
    tag_ids: NotRequired[QueryParamTagIdsTypedDict]
    r"""The tag IDs to filter the links by."""
    tag_names: NotRequired[QueryParamTagNamesTypedDict]
    r"""The unique name of the tags assigned to the short link (case insensitive)."""
    search: NotRequired[str]
    r"""The search term to filter the links by. The search term will be matched against the short link slug and the destination url."""
    user_id: NotRequired[str]
    r"""The user ID to filter the links by."""
    show_archived: NotRequired[bool]
    r"""Whether to include archived links in the response. Defaults to `false` if not provided."""
    with_tags: NotRequired[bool]
    r"""DEPRECATED. Filter for links that have at least one tag assigned to them."""
    sort: NotRequired[Sort]
    r"""The field to sort the links by. The default is `createdAt`, and sort order is always descending."""
    page: NotRequired[float]
    r"""The page number for pagination."""
    page_size: NotRequired[float]
    r"""The number of items per page."""


class GetLinksRequest(BaseModel):
    domain: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The domain to filter the links by. E.g. `ac.me`. If not provided, all links for the workspace will be returned."""

    tag_id: Annotated[
        Optional[str],
        pydantic.Field(alias="tagId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The tag ID to filter the links by. This field is deprecated – use `tagIds` instead."""

    tag_ids: Annotated[
        Optional[QueryParamTagIds],
        pydantic.Field(alias="tagIds"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The tag IDs to filter the links by."""

    tag_names: Annotated[
        Optional[QueryParamTagNames],
        pydantic.Field(alias="tagNames"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The unique name of the tags assigned to the short link (case insensitive)."""

    search: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The search term to filter the links by. The search term will be matched against the short link slug and the destination url."""

    user_id: Annotated[
        Optional[str],
        pydantic.Field(alias="userId"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The user ID to filter the links by."""

    show_archived: Annotated[
        Optional[bool],
        pydantic.Field(alias="showArchived"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = True
    r"""Whether to include archived links in the response. Defaults to `false` if not provided."""

    with_tags: Annotated[
        Optional[bool],
        pydantic.Field(alias="withTags"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = True
    r"""DEPRECATED. Filter for links that have at least one tag assigned to them."""

    sort: Annotated[
        Optional[Sort],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = Sort.CREATED_AT
    r"""The field to sort the links by. The default is `createdAt`, and sort order is always descending."""

    page: Annotated[
        Optional[float],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""The page number for pagination."""

    page_size: Annotated[
        Optional[float],
        pydantic.Field(alias="pageSize"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 100
    r"""The number of items per page."""


class GetLinksResponseTypedDict(TypedDict):
    result: List[components_linkschema.LinkSchemaTypedDict]


class GetLinksResponse(BaseModel):
    next: Callable[[], Optional[GetLinksResponse]]

    result: List[components_linkschema.LinkSchema]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub import utils
from dub.types import BaseModel
from enum import Enum
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class InternalServerErrorCode(str, Enum):
    r"""A short code indicating the error code returned."""

    INTERNAL_SERVER_ERROR = "internal_server_error"


class InternalServerErrorErrorTypedDict(TypedDict):
    code: InternalServerErrorCode
    r"""A short code indicating the error code returned."""
    message: str
    r"""A human readable explanation of what went wrong."""
    doc_url: NotRequired[str]
    r"""A link to our documentation with more details about this error code"""


class InternalServerErrorError(BaseModel):
    code: InternalServerErrorCode
    r"""A short code indicating the error code returned."""

    message: str
    r"""A human readable explanation of what went wrong."""

    doc_url: Optional[str] = None
    r"""A link to our documentation with more details about this error code"""


class InternalServerErrorData(BaseModel):
    error: InternalServerErrorError


class InternalServerError(Exception):
    r"""The server has encountered a situation it does not know how to handle."""

    data: InternalServerErrorData

    def __init__(self, data: InternalServerErrorData):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(self.data, InternalServerErrorData)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dub.types import BaseModel
from dub.utils import FieldMetadata, PathParamMetadata
from typing_extensions import Annotated, TypedDict


class DeleteDomainRequestTypedDict(TypedDict):
    slug: str
    r"""The domain name."""


class DeleteDomainRequest(BaseModel):
    slug: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The domain name."""


class DeleteDomainResponseBodyTypedDict(TypedDict):
    r"""The domain was deleted."""

    slug: str
    r"""The domain name."""


class DeleteDomainResponseBody(BaseModel):
    r"""The domain was deleted."""

    slug: str
    r"""The domain name."""
